@@ -22,6 +22,8 @@ class Constraint:
         if self.save == True:
             self.constraint_equations = []
 
+        self.posg = 0
+
     def g(self):
         pass
 
@@ -41,22 +43,63 @@ class Constraint:
         ey = np.array([0, 1, 0])
         m2 = np.dot(Constraint.tilde(axis), ex)
         if np.dot(m2.T, m2) == 0:
-            m2 = np.dot(Constraint.tilde(axis), ey)
+            m2 = np.abs(np.dot(Constraint.tilde(axis), ey))
 
-        n2 = np.dot(Constraint.tilde(axis), m2)
+        n2 = np.abs(np.dot(Constraint.tilde(axis), m2))
         return m2, n2
 
+    @staticmethod
+    def rotation_matrix(pE):
+        e0, e1, e2, e3 = pE
+        G = np.array(
+            [[-e1, e0, -e3, e2], [-e2, e3, e0, -e1], [-e3, -e2, e1, e0]])
+        L = np.array(
+            [[-e1, e0, e3, -e2], [-e2, -e3, e0, e1], [-e3, e2, -e1, e0]])
+        A0K = np.dot(G, L.transpose())
+        return A0K, G, L
+
 
 class Joint(Constraint):
     dof = 3
 
-    def __init__(self, p1: bodies.Marker, p2: bodies.Marker, rotational_damping=0):
-        Constraint.__init__(self, p1, p2)
+    def __init__(self, p1: bodies.Marker, p2: bodies.Marker, rotational_damping=0, save=False, name='Joint'):
+        Constraint.__init__(self, p1, p2, save)
         self.damp = rotational_damping
+        self.qz = np.zeros(self.dof)
+        self.name = name
+
+    def update_constraint_force(self, c_global):
+        self.qz = c_global[self.posg:self.posg + self.dof]
+
+    def get_constraint_force(self):
+        data = {
+            f'{self.name}_Fg1': self.qz[0],
+            f'{self.name}_Fg2': self.qz[1],
+            f'{self.name}_Fg3': self.qz[2],
+        }
+        return data
 
     def g(self):
         gy = self.body2.r0S0 + np.dot(self.body2.A0K, self.p2.rSPK) - \
             (self.body1.r0S0 + np.dot(self.body1.A0K, self.p1.rSPK))
+        # print(gy)
+        return gy
+
+    def g_lambda(self, y1, y2):
+        # Location and Euler Parameter
+        r010 = y1[0:3]
+        pE1 = y1[3:7]
+        r020 = y2[0:3]
+        pE2 = y2[3:7]
+
+        # Rotation matrices
+        A01, _, _ = self.rotation_matrix(pE1)
+        A02, _, _ = self.rotation_matrix(pE2)
+
+        # Constraint equation
+        gy = r020 + np.dot(A02, self.p2.rSPK) - \
+            (r010 + np.dot(A01, self.p1.rSPK))
+
         return gy
 
     def jacobian(self):
@@ -67,13 +110,11 @@ class Joint(Constraint):
         # first derivation of angular velocity part
         r1B1 = self.p1.rSPK
         r2B2 = self.p2.rSPK
-        r2B2 = self.p2.rSPK
 
         Jg1[:, 0:3] = E
         Jg1[:, 3:6] = np.dot(self.body1.A0K, bodies.tilde(r1B1).transpose())
         Jg2[:, 0:3] = -E
         Jg2[:, 3:6] = -np.dot(self.body2.A0K, bodies.tilde(r2B2).transpose())
-
         return Jg1, Jg2
 
     def djacobian_z(self):
@@ -126,11 +167,44 @@ class Hinge(Constraint):
         m2 = self.m2
         n2 = self.n2
 
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+        A12 = np.dot(A01.T, A02)
+
         # Constraint equations
         gy1 = self.body2.r0S0 + np.dot(self.body2.A0K, self.p2.rSPK) - \
             (self.body1.r0S0 + np.dot(self.body1.A0K, self.p1.rSPK))
-        gy2 = np.dot(m2.T, u2)
-        gy3 = np.dot(n2.T, u2)
+        gy2 = np.dot(np.dot(A12, m2.T).T, u2)
+        gy3 = np.dot(np.dot(A12, n2.T).T, u2)
+        return np.hstack((gy1, gy2, gy3))
+
+    def g_lambda(self, y1, y2):
+        # Location and Euler Parameter
+        r010 = y1[0:3]
+        pE1 = y1[3:7]
+        r020 = y2[0:3]
+        pE2 = y2[3:7]
+
+        # Rotation matrices
+        A01, _, _ = self.rotation_matrix(pE1)
+        A02, _, _ = self.rotation_matrix(pE2)
+        A12 = np.dot(A01.T, A02)
+
+        # Axis for rotational constraints
+        s = np.dot(A01, self.ey)
+        a = np.dot(A02, self.ex)
+        b = np.dot(A02, self.ez)
+
+        u2 = self.rotation_axis
+
+        m2 = self.m2
+        n2 = self.n2
+
+        # Constraint equations
+        gy1 = r020 + np.dot(A02, self.p2.rSPK) - \
+            (r010 + np.dot(A01, self.p1.rSPK))
+        gy2 = np.dot(np.dot(A12, m2.T).T, u2)
+        gy3 = np.dot(np.dot(A12, n2.T).T, u2)
         return np.hstack((gy1, gy2, gy3))
 
     def jacobian(self):
@@ -195,6 +269,561 @@ class Hinge(Constraint):
         return Jgpz
 
 
+class Fixed(Constraint):
+    dof = 6
+
+    def __init__(self, p1: bodies.Marker, p2: bodies.Marker, name="Fix", save=False):
+        Constraint.__init__(self, p1, p2, save)
+
+        self.ex = np.array([1, 0, 0])
+        self.ey = np.array([0, 1, 0])
+        self.ez = np.array([0, 0, 1])
+
+        self.name = name
+
+        self.Jg1 = np.array([[1, 0, 0, 0, 0, 0],
+                             [0, 1, 0, 0, 0, 0],
+                             [0, 0, 1, 0, 0, 0],
+                             [0, 0, 0, 0, 0, 0],
+                             [0, 0, 0, 0, 0, 0],
+                             [0, 0, 0, 0, 0, 0]])
+
+        self.Jg2 = np.array([[-1, 0, 0, 0, 0, 0],
+                             [0, -1, 0, 0, 0, 0],
+                             [0, 0, -1, 0, 0, 0],
+                             [0, 0, 0, 0, 0, 0],
+                             [0, 0, 0, 0, 0, 0],
+                             [0, 0, 0, 0, 0, 0]])
+
+    def g(self):
+        # Axis for rotational constraints
+        s = np.dot(self.body1.A0K, self.ey)
+        a = np.dot(self.body2.A0K, self.ex)
+        b = np.dot(self.body2.A0K, self.ez)
+
+        u2 = self.rotation_axis
+
+        m2 = self.m2
+        n2 = self.n2
+
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+        A12 = np.dot(A01.T, A02)
+
+        # Constraint equations
+        gy1 = self.body2.r0S0 + np.dot(self.body2.A0K, self.p2.rSPK) - \
+            (self.body1.r0S0 + np.dot(self.body1.A0K, self.p1.rSPK))
+        gy2 = np.dot(np.dot(A12, m2.T).T, u2)
+        gy3 = np.dot(np.dot(A12, n2.T).T, u2)
+        return np.hstack((gy1, gy2, gy3))
+
+    def g_lambda(self, y1, y2):
+        # Location and Euler Parameter
+        r010 = y1[0:3]
+        pE1 = y1[3:7]
+        r020 = y2[0:3]
+        pE2 = y2[3:7]
+
+        # Rotation matrices
+        A01, _, _ = self.rotation_matrix(pE1)
+        A02, _, _ = self.rotation_matrix(pE2)
+        A12 = np.dot(A01.T, A02)
+
+        # Axis for rotational constraints
+        s = np.dot(A01, self.ey)
+        a = np.dot(A02, self.ex)
+        b = np.dot(A02, self.ez)
+
+        u2 = self.rotation_axis
+
+        m2 = self.m2
+        n2 = self.n2
+
+        # Constraint equations
+        gy1 = r020 + np.dot(A02, self.p2.rSPK) - \
+            (r010 + np.dot(A01, self.p1.rSPK))
+        gy2 = np.dot(np.dot(A12, m2.T).T, u2)
+        gy3 = np.dot(np.dot(A12, n2.T).T, u2)
+        return np.hstack((gy1, gy2, gy3))
+
+    def jacobian(self):
+        E = np.identity(3)
+        Z1 = np.zeros([1, 3])
+
+        # rotation axis
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+        A12 = np.dot(A01.T, A02)
+
+        u2 = self.rotation_axis
+
+        m2 = self.m2
+        n2 = self.n2
+
+        # first derivation of angular velocity part
+        r1B1 = self.p1.rSPK
+        r2B2 = self.p2.rSPK
+
+        # self.Jg1[0:3, 3:] = np.dot(A01, bodies.tilde(r1B1).T)
+        # self.Jg1[3, 3:] = -np.dot(self.tilde(m2).T, u2)
+        # self.Jg1[4, 3:] = -np.dot(self.tilde(n2).T, u2)
+        #
+        # self.Jg2[0:3, 3:] = -np.dot(A02, bodies.tilde(r2B2).T)
+        # self.Jg2[3, 3:] = np.dot(A12.T, np.dot(self.tilde(m2).T, u2))
+        # self.Jg2[4, 3:] = np.dot(A12.T, np.dot(self.tilde(n2).T, u2))
+
+        Jg1 = np.block([[E, np.dot(A01, bodies.tilde(r1B1).T)],
+                        [Z1, -np.dot(self.tilde(m2).T, u2)],
+                        [Z1, -np.dot(self.tilde(n2).T, u2)]])
+
+        Jg2 = np.block([[-E, -np.dot(A02, bodies.tilde(r2B2).T)],
+                        [Z1, np.dot(A12.T, np.dot(self.tilde(m2).T, u2))],
+                        [Z1, np.dot(A12.T, np.dot(self.tilde(n2).T, u2))]])
+
+        return Jg1, Jg2
+
+    def djacobian_z(self):
+        # rotation axis
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+
+        u2 = self.rotation_axis
+        m2 = self.m2
+        n2 = self.n2
+
+        om1 = self.body1.om0KK
+        om2 = self.body2.om0KK
+
+        r1B1 = self.p1.rSPK
+        r2B2 = self.p2.rSPK
+
+        A01d = self.body1.derivation_rotation_matrix
+        A02d = self.body2.derivation_rotation_matrix
+        A12dT = np.dot(A01d.transpose(), A02) + np.dot(A01.transpose(), A02d)
+
+        Jgpz = np.block([-np.dot(A01d, np.dot(self.tilde(r1B1).T, om1)) + np.dot(A02d, np.dot(self.tilde(r2B2).T, om2)),
+                         np.dot(np.dot(A12dT, np.dot(self.tilde(m2).T, u2)), om2),
+                         np.dot(np.dot(A12dT, np.dot(self.tilde(n2).T, u2)), om2)])
+
+        return Jgpz
+
+
+class Joint_reduced(Constraint):
+    dof = 2
+
+    def __init__(self, p1: bodies.Marker, p2: bodies.Marker, rotational_damping=0, save=False, name='Joint'):
+        Constraint.__init__(self, p1, p2, save)
+        self.damp = rotational_damping
+        self.qz = np.zeros(self.dof)
+        self.name = name
+
+    def update_constraint_force(self, c_global):
+        self.qz = c_global[self.posg:self.posg + self.dof]
+
+    def get_constraint_force(self):
+        data = {
+            f'{self.name}_Fg1': self.qz[0],
+            f'{self.name}_Fg2': self.qz[1],
+            f'{self.name}_Fg3': self.qz[2],
+        }
+        return data
+
+    def g(self):
+        gy = self.body2.r0S0 + np.dot(self.body2.A0K, self.p2.rSPK) - \
+            (self.body1.r0S0 + np.dot(self.body1.A0K, self.p1.rSPK))
+        # print(gy)
+        return gy
+
+    def g_lambda(self, y1, y2):
+        # Location and Euler Parameter
+        r010 = y1[0:3]
+        pE1 = y1[3:7]
+        r020 = y2[0:3]
+        pE2 = y2[3:7]
+
+        # Rotation matrices
+        A01, _, _ = self.rotation_matrix(pE1)
+        A02, _, _ = self.rotation_matrix(pE2)
+
+        # Constraint equation
+        gy = r020 + np.dot(A02, self.p2.rSPK) - \
+            (r010 + np.dot(A01, self.p1.rSPK))
+
+        return gy
+
+    def jacobian(self):
+        Jg1 = np.zeros([3, 6])
+        Jg2 = np.zeros([3, 6])
+        E = np.identity(3)
+
+        # first derivation of angular velocity part
+        r1B1 = self.p1.rSPK
+        r2B2 = self.p2.rSPK
+        r2B2 = self.p2.rSPK
+
+        Jg1[:, 0:3] = E
+        Jg1[:, 3:6] = np.dot(self.body1.A0K, bodies.tilde(r1B1).transpose())
+        Jg2[:, 0:3] = -E
+        Jg2[:, 3:6] = -np.dot(self.body2.A0K, bodies.tilde(r2B2).transpose())
+
+        Jg1 = np.delete(Jg1, 1, 0)
+        Jg2 = np.delete(Jg2, 1, 0)
+
+        return Jg1, Jg2
+
+    def djacobian_z(self):
+        om1 = self.body1.om0KK
+        om2 = self.body2.om0KK
+        Jgpz = np.dot(self.body2.A0K, (cross(om2, cross(om2, self.p2.rSPK)))) - \
+            np.dot(self.body1.A0K, (cross(om1, cross(om1, self.p1.rSPK))))
+
+        Jgpz = np.delete(Jgpz, 1, 0)
+
+        return Jgpz
+
+
+class Prismatic(Constraint):
+    dof = 5
+
+    def __init__(self, p1: bodies.Marker, p2: bodies.Marker, axis, damp=0, name="Prismatic", save=False):
+        Constraint.__init__(self, p1, p2, save)
+        self.slide_axis = np.array(axis)
+
+        self.ex = np.array([1, 0, 0])
+        self.ey = np.array([0, 1, 0])
+        self.ez = np.array([0, 0, 1])
+
+        self.p11, self.p21 = self.get_perpendicular_axis(self.slide_axis)
+
+        # Initial Connection Array
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+        r1P11 = self.p1.rSPK
+        r2P22 = self.p2.rSPK
+        r0P10 = self.body1.r0S0 + np.dot(A01, r1P11)
+        r0P20 = self.body2.r0S0 + np.dot(A02, r2P22)
+        self.r0 = r0P20 - r0P10
+
+        self.damp = damp
+
+        self.name = name
+
+        self.Jg1 = np.zeros([5, 6])
+        self.Jg2 = np.zeros([5, 6])
+
+    def g(self):
+        # Rotation
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+
+        # Global perpendicular arrays
+        p10 = np.dot(A01, self.p11)
+        p20 = np.dot(A01, self.p21)
+
+        # Global unit arrays
+        ex10 = np.dot(A01, self.ex)
+        ey10 = np.dot(A01, self.ey)
+        ez10 = np.dot(A01, self.ez)
+        ex20 = np.dot(A02, self.ex)
+        ey20 = np.dot(A02, self.ey)
+        ez20 = np.dot(A02, self.ez)
+
+        # Connection array
+        r1P11 = self.p1.rSPK
+        r2P22 = self.p2.rSPK
+        r0P10 = self.body1.r0S0 + np.dot(A01, r1P11)
+        r0P20 = self.body2.r0S0 + np.dot(A02, r2P22)
+        rP1P20 = r0P20 - r0P10 - self.r0
+
+        # Two connection constraints
+        g1 = np.dot(p10.T, rP1P20)
+        g2 = np.dot(p20.T, rP1P20)
+
+        # 3 Perpendicular constraints
+        g3 = np.dot(ex10.T, ey20)
+        g4 = np.dot(ey10.T, ez20)
+        g5 = np.dot(ez10.T, ex20)
+
+        return [g1, g2, g3, g4, g5]
+
+    def jacobian(self):
+        # Rotation
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+
+        # Global perpendicular arrays
+        p10 = np.dot(A01, self.p11)
+        p20 = np.dot(A01, self.p21)
+
+        # Global unit arrays
+        ex10 = np.dot(A01, self.ex)
+        ey10 = np.dot(A01, self.ey)
+        ez10 = np.dot(A01, self.ez)
+        ex20 = np.dot(A02, self.ex)
+        ey20 = np.dot(A02, self.ey)
+        ez20 = np.dot(A02, self.ez)
+
+        # Connection array
+        r120 = self.body2.r0S0 - self.body1.r0S0
+
+        # Two connection constraints
+        self.Jg1[0, 0:3] = -p10.T
+        self.Jg1[0, 3:6] = -np.dot(r120.T, np.dot(self.tilde(p10), A01))
+        self.Jg1[1, 0:3] = -p20.T
+        self.Jg1[1, 3:6] = -np.dot(r120.T, np.dot(self.tilde(p20), A01))
+
+        self.Jg2[0, 0:3] = p10.T
+        self.Jg2[1, 0:3] = p20.T
+
+        # Three perpendicular constraints
+        self.Jg1[2, 3:6] = -np.dot(ey20.T, np.dot(self.tilde(ex10), A01))
+        self.Jg1[3, 3:6] = -np.dot(ez20.T, np.dot(self.tilde(ey10), A01))
+        self.Jg1[4, 3:6] = -np.dot(ex20.T, np.dot(self.tilde(ez10), A01))
+
+        self.Jg2[2, 3:6] = -np.dot(ex10.T, np.dot(self.tilde(ey20), A02))
+        self.Jg2[3, 3:6] = -np.dot(ey10.T, np.dot(self.tilde(ez20), A02))
+        self.Jg2[4, 3:6] = -np.dot(ez10.T, np.dot(self.tilde(ex20), A02))
+
+        return self.Jg1, self.Jg2
+
+    def djacobian_z(self):
+         # Rotation
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+
+        # Global perpendicular arrays
+        p10 = np.dot(A01, self.p11)
+        p20 = np.dot(A01, self.p21)
+
+        # Global unit arrays
+        ex10 = np.dot(A01, self.ex)
+        ey10 = np.dot(A01, self.ey)
+        ez10 = np.dot(A01, self.ez)
+        ex20 = np.dot(A02, self.ex)
+        ey20 = np.dot(A02, self.ey)
+        ez20 = np.dot(A02, self.ez)
+
+        # Connection array
+        r120 = self.body2.r0S0 - self.body1.r0S0
+
+        # Derivative of rotation matrix
+        A01d = self.body1.A0Kd
+        A02d = self.body2.A0Kd
+
+        # Derivative of global perpendicular arrays
+        p10d = np.dot(A01d, self.p11)
+        p20d = np.dot(A01d, self.p21)
+
+        # Derivative of global unit arrays
+        ex10d = np.dot(A01d, self.ex)
+        ey10d = np.dot(A01d, self.ey)
+        ez10d = np.dot(A01d, self.ez)
+        ex20d = np.dot(A02d, self.ex)
+        ey20d = np.dot(A02d, self.ey)
+        ez20d = np.dot(A02d, self.ez)
+
+        # Derivative of connection array
+        r120d = self.body2.v0S0 - self.body1.v0S0
+
+        # Global angular velocities
+        om010 = np.dot(A01, self.body1.om0KK)
+        om020 = np.dot(A02, self.body2.om0KK)
+
+        # Jgp*z
+        Jgpz = np.zeros(5)
+        Jgpz[0] = - np.dot(p10.T, np.dot(self.tilde(om010), r120d)) \
+            - np.dot(r120.T, np.dot(self.tilde(om010), p10d)) \
+            - 2 * np.dot(r120d.T, p10d)
+        Jgpz[1] = - np.dot(p20.T, np.dot(self.tilde(om010), r120d)) \
+            - np.dot(r120.T, np.dot(self.tilde(om010), p20d)) \
+            - 2 * np.dot(r120d.T, p20d)
+        Jgpz[2] = - np.dot(ex10.T, np.dot(self.tilde(om020), ey20d)) \
+            - np.dot(ey20.T, np.dot(self.tilde(om010), ex10d)) \
+            - 2 * np.dot(ey20d.T, ex10d)
+        Jgpz[3] = - np.dot(ey10.T, np.dot(self.tilde(om020), ez20d)) \
+            - np.dot(ez20.T, np.dot(self.tilde(om010), ey10d)) \
+            - 2 * np.dot(ez20d.T, ey10d)
+        Jgpz[4] = - np.dot(ez10.T, np.dot(self.tilde(om020), ex20d)) \
+            - np.dot(ex20.T, np.dot(self.tilde(om010), ez10d)) \
+            - 2 * np.dot(ex20d.T, ez10d)
+
+        return Jgpz
+
+
+class Prismatic_reduced(Constraint):
+    dof = 4
+
+    def __init__(self, p1: bodies.Marker, p2: bodies.Marker, axis, damp=0, name="Prismatic", save=False):
+        Constraint.__init__(self, p1, p2, save)
+        self.slide_axis = np.array(axis)
+
+        self.ex = np.array([1, 0, 0])
+        self.ey = np.array([0, 1, 0])
+        self.ez = np.array([0, 0, 1])
+
+        self.p11, self.p21 = self.get_perpendicular_axis(self.slide_axis)
+
+        # Initial Connection Array
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+        r1P11 = self.p1.rSPK
+        r2P22 = self.p2.rSPK
+        r0P10 = self.body1.r0S0 + np.dot(A01, r1P11)
+        r0P20 = self.body2.r0S0 + np.dot(A02, r2P22)
+        self.r0 = r0P20 - r0P10
+
+        self.damp = damp
+
+        self.name = name
+
+        self.Jg1 = np.zeros([5, 6])
+        self.Jg2 = np.zeros([5, 6])
+
+    def g(self):
+        # Rotation
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+
+        # Global perpendicular arrays
+        p10 = np.dot(A01, self.p11)
+        p20 = np.dot(A01, self.p21)
+
+        # Global unit arrays
+        ex10 = np.dot(A01, self.ex)
+        ey10 = np.dot(A01, self.ey)
+        ez10 = np.dot(A01, self.ez)
+        ex20 = np.dot(A02, self.ex)
+        ey20 = np.dot(A02, self.ey)
+        ez20 = np.dot(A02, self.ez)
+
+        # Connection array
+        r1P11 = self.p1.rSPK
+        r2P22 = self.p2.rSPK
+        r0P10 = self.body1.r0S0 + np.dot(A01, r1P11)
+        r0P20 = self.body2.r0S0 + np.dot(A02, r2P22)
+        rP1P20 = r0P20 - r0P10 - self.r0
+
+        # Two connection constraints
+        g1 = np.dot(p10.T, rP1P20)
+        g2 = np.dot(p20.T, rP1P20)
+
+        # 3 Perpendicular constraints
+        g3 = np.dot(ex10.T, ey20)
+        g4 = np.dot(ey10.T, ez20)
+        g5 = np.dot(ez10.T, ex20)
+
+        return [g1, g2, g3, g4, g5]
+
+    def jacobian(self):
+        # Rotation
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+
+        # Global perpendicular arrays
+        p10 = np.dot(A01, self.p11)
+        p20 = np.dot(A01, self.p21)
+
+        # Global unit arrays
+        ex10 = np.dot(A01, self.ex)
+        ey10 = np.dot(A01, self.ey)
+        ez10 = np.dot(A01, self.ez)
+        ex20 = np.dot(A02, self.ex)
+        ey20 = np.dot(A02, self.ey)
+        ez20 = np.dot(A02, self.ez)
+
+        # Connection array
+        r120 = self.body2.r0S0 - self.body1.r0S0
+
+        Jg1 = self.Jg1
+        Jg2 = self.Jg2
+        # Two connection constraints
+        Jg1[0, 0:3] = -p10.T
+        Jg1[0, 3:6] = -np.dot(r120.T, np.dot(self.tilde(p10), A01))
+        Jg1[1, 0:3] = -p20.T
+        Jg1[1, 3:6] = -np.dot(r120.T, np.dot(self.tilde(p20), A01))
+
+        Jg2[0, 0:3] = p10.T
+        Jg2[1, 0:3] = p20.T
+
+        # Three perpendicular constraints
+        Jg1[2, 3:6] = -np.dot(ey20.T, np.dot(self.tilde(ex10), A01))
+        Jg1[3, 3:6] = -np.dot(ez20.T, np.dot(self.tilde(ey10), A01))
+        Jg1[4, 3:6] = -np.dot(ex20.T, np.dot(self.tilde(ez10), A01))
+
+        Jg2[2, 3:6] = -np.dot(ex10.T, np.dot(self.tilde(ey20), A02))
+        Jg2[3, 3:6] = -np.dot(ey10.T, np.dot(self.tilde(ez20), A02))
+        Jg2[4, 3:6] = -np.dot(ez10.T, np.dot(self.tilde(ex20), A02))
+
+        Jg1 = np.delete(Jg1, 0, 0)
+        Jg2 = np.delete(Jg2, 0, 0)
+        return Jg1, Jg2
+
+    def djacobian_z(self):
+         # Rotation
+        A01 = self.body1.A0K
+        A02 = self.body2.A0K
+
+        # Global perpendicular arrays
+        p10 = np.dot(A01, self.p11)
+        p20 = np.dot(A01, self.p21)
+
+        # Global unit arrays
+        ex10 = np.dot(A01, self.ex)
+        ey10 = np.dot(A01, self.ey)
+        ez10 = np.dot(A01, self.ez)
+        ex20 = np.dot(A02, self.ex)
+        ey20 = np.dot(A02, self.ey)
+        ez20 = np.dot(A02, self.ez)
+
+        # Connection array
+        r120 = self.body2.r0S0 - self.body1.r0S0
+
+        # Derivative of rotation matrix
+        A01d = self.body1.A0Kd
+        A02d = self.body2.A0Kd
+
+        # Derivative of global perpendicular arrays
+        p10d = np.dot(A01d, self.p11)
+        p20d = np.dot(A01d, self.p21)
+
+        # Derivative of global unit arrays
+        ex10d = np.dot(A01d, self.ex)
+        ey10d = np.dot(A01d, self.ey)
+        ez10d = np.dot(A01d, self.ez)
+        ex20d = np.dot(A02d, self.ex)
+        ey20d = np.dot(A02d, self.ey)
+        ez20d = np.dot(A02d, self.ez)
+
+        # Derivative of connection array
+        r120d = self.body2.v0S0 - self.body1.v0S0
+
+        # Global angular velocities
+        om010 = np.dot(A01, self.body1.om0KK)
+        om020 = np.dot(A02, self.body2.om0KK)
+
+        # Jgp*z
+        Jgpz = np.zeros(5)
+        Jgpz[0] = - np.dot(p10.T, np.dot(self.tilde(om010), r120d)) \
+            - np.dot(r120.T, np.dot(self.tilde(om010), p10d)) \
+            - 2 * np.dot(r120d.T, p10d)
+        Jgpz[1] = - np.dot(p20.T, np.dot(self.tilde(om010), r120d)) \
+            - np.dot(r120.T, np.dot(self.tilde(om010), p20d)) \
+            - 2 * np.dot(r120d.T, p20d)
+        Jgpz[2] = - np.dot(ex10.T, np.dot(self.tilde(om020), ey20d)) \
+            - np.dot(ey20.T, np.dot(self.tilde(om010), ex10d)) \
+            - 2 * np.dot(ey20d.T, ex10d)
+        Jgpz[3] = - np.dot(ey10.T, np.dot(self.tilde(om020), ez20d)) \
+            - np.dot(ez20.T, np.dot(self.tilde(om010), ey10d)) \
+            - 2 * np.dot(ez20d.T, ey10d)
+        Jgpz[4] = - np.dot(ez10.T, np.dot(self.tilde(om020), ex20d)) \
+            - np.dot(ex20.T, np.dot(self.tilde(om010), ez10d)) \
+            - 2 * np.dot(ex20d.T, ez10d)
+
+        Jgpz = np.delete(Jgpz, 0, 0)
+
+        return Jgpz
+
+
 class PlanarJoint(Constraint):
     dof = 3
 
@@ -407,7 +1036,7 @@ class PrismaticHinge(Hinge):
         self.translation_axis = np.array(translation_axis)
 
         # Perpendicular axis
-        self.a1,  self.b1 = self.get_perpendicular_axis(translation_axis)
+        self.a1, self.b1 = self.get_perpendicular_axis(translation_axis)
 
     def g(self):
         Z1 = np.zeros([1, 3])
